<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2009-2010 Pixware SARL. All rights reserved.
|
| Author: Hussein Shafie
|
| This file is part of the XMLmind DITA Converter project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:u="http://www.xmlmind.com/namespace/ditac"
                xmlns:Image="java:com.xmlmind.ditac.xslt.Image"
                exclude-result-prefixes="xs u Image"
                version="2.0">

  <!-- indexOfNode ======================================================= -->

  <xsl:function name="u:indexOfNode" as="xs:integer*">
    <xsl:param name="nodeSequence" as="node()*"/>
    <xsl:param name="searchedNode" as="node()"/>

    <xsl:for-each select="$nodeSequence">
      <xsl:if test=". is $searchedNode">
        <xsl:sequence select="position()"/>
      </xsl:if>
    </xsl:for-each>
  </xsl:function>

  <!-- classToElementName ================================================ -->

  <xsl:function name="u:classToElementName" as="xs:string">
    <xsl:param name="class" as="xs:string"/>

    <xsl:sequence select="tokenize(normalize-space($class), '/|\s+')[last()]"/>
  </xsl:function>

  <!-- imageWidth ======================================================== -->

  <xsl:template name="imageWidth">
    <xsl:param name="path" select="''"/>

    <xsl:sequence select="Image:getWidth(resolve-uri($path, base-uri()))"/>
  </xsl:template>

  <!-- noteType ========================================================== -->

  <xsl:template name="noteType">
    <xsl:choose>
      <xsl:when test="@type = 'other' and @othertype">
        <xsl:value-of select="normalize-space(@othertype)"/>
      </xsl:when>
      <xsl:when test="@type and @type != 'other'">
        <xsl:value-of select="@type"/>
      </xsl:when>
      <xsl:otherwise>note</xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Table processing utilities ======================================== -->

  <xsl:template name="processCellContents">
    <xsl:choose>
      <xsl:when test="exists(./text()|./*)">
        <xsl:apply-templates/>
      </xsl:when>
      <xsl:otherwise>&#xA0;</xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:function name="u:entryRowSpan" as="xs:double">
    <xsl:param name="entry" as="element()"/>

    <xsl:for-each select="$entry">
      <xsl:choose>
        <!-- number() returns NaN instead of raising an error. -->
        <xsl:when test="number(@morerows) gt 0">
          <xsl:sequence select="number(@morerows) + 1"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:sequence select="1"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
  </xsl:function>

  <xsl:function name="u:entryColSpan" as="xs:integer">
    <xsl:param name="entry" as="element()"/>

    <xsl:for-each select="$entry">
      <xsl:variable name="namest" select="@namest"/>
      <xsl:variable name="nameend" select="@nameend"/>

      <xsl:choose>
        <xsl:when test="$namest and $nameend">
          <xsl:variable name="colst" 
            select="(ancestor::*[contains(@class,' topic/tgroup ')]/*[contains(@class,' topic/colspec ') and (@colname = $namest)])[1]"/>

          <xsl:variable name="colend" 
            select="(ancestor::*[contains(@class,' topic/tgroup ')]/*[contains(@class,' topic/colspec ') and (@colname = $nameend)])[1]"/>
          
          <xsl:choose>
            <xsl:when test="exists($colst) and exists($colend)">
              <xsl:variable name="colnumst" 
                            select="u:colspecNumber($colst)"/>
              <xsl:variable name="colnumend" 
                            select="u:colspecNumber($colend)"/>

              <xsl:choose>
                <xsl:when test="$colnumend gt $colnumst">
                  <xsl:sequence select="$colnumend - $colnumst + 1"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:sequence select="1"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
              <xsl:sequence select="1"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <xsl:sequence select="1"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
  </xsl:function>

  <xsl:function name="u:colspecNumber" as="xs:integer">
    <xsl:param name="colspec" as="element()"/>

    <xsl:for-each select="$colspec">
      <xsl:sequence select="count(preceding-sibling::*[contains(@class,' topic/colspec ')]) + 1"/>
    </xsl:for-each>
  </xsl:function>

  <xsl:function name="u:entryColspec" as="element()?">
    <xsl:param name="entry" as="element()"/>

    <xsl:for-each select="$entry">
      <xsl:variable name="colname" select="@colname"/>
      <xsl:variable name="namest" select="@namest"/>

      <xsl:choose>
        <xsl:when test="$colname">
          <xsl:sequence
            select="(ancestor::*[contains(@class,' topic/tgroup ')]/*[contains(@class,' topic/colspec ') and (@colname = $colname)])[1]"/>
        </xsl:when>
        <xsl:when test="$namest">
          <xsl:sequence
            select="(ancestor::*[contains(@class,' topic/tgroup ')]/*[contains(@class,' topic/colspec ') and (@colname = $namest)])[1]"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:sequence select="()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
  </xsl:function>

  <!-- localize ========================================================== -->

  <xsl:param name="messageFileNames"
             select="'en', 'fr', 'de', 'es', 'ru', 'cs'"/>

  <xsl:template name="localize">
    <xsl:param name="message" select="''"/>

    <xsl:variable name="lang">
      <xsl:call-template name="lang"/>
    </xsl:variable>

    <xsl:variable name="messageFileName">
      <xsl:choose>
        <xsl:when test="index-of($messageFileNames, $lang) ge 1">
          <xsl:value-of select="$lang"/>
        </xsl:when>
        <xsl:otherwise>en</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="messageFile"
                  select="concat('messages/', $messageFileName, '.xml')"/>

    <!-- Hope that documents loaded using document() are cached. -->
    <xsl:variable name="messages"
                  select="document($messageFile)/messages"/>

    <xsl:choose>
      <xsl:when test="$messages/message[@name=$message]">
        <xsl:value-of select="string($messages/message[@name=$message])"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$message"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="lang">
    <xsl:variable name="lang"
      select="lower-case(if (ancestor-or-self::*/@xml:lang) then 
                             string((ancestor-or-self::*/@xml:lang)[last()])
                         else 
                             'en')"/>
    
    <xsl:choose>
      <xsl:when test="contains($lang, '-')">
        <xsl:value-of select="substring-before($lang, '-')"/>
      </xsl:when>
      <xsl:when test="contains($lang, '_')">
        <xsl:value-of select="substring-before($lang, '_')"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$lang"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Function variant -->

  <xsl:function name="u:localize" as="xs:string">
    <xsl:param name="message" as="xs:string"/>
    <xsl:param name="context" as="element()"/>

    <xsl:for-each select="$context">
      <xsl:call-template name="localize">
        <xsl:with-param name="message" select="$message"/>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:function>

</xsl:stylesheet>
