<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2009-2010 Pixware SARL. All rights reserved.
|
| Author: Hussein Shafie
|
| This file is part of the XMLmind DITA Converter project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns="http://www.w3.org/1999/xhtml"
                xmlns:u="http://www.xmlmind.com/namespace/ditac"
                exclude-result-prefixes="xs u"
                version="2.0">
  
  <!-- table ============================================================= -->

  <xsl:template match="*[contains(@class,' topic/table ')]">
    <div>
      <xsl:call-template name="commonAttributes"/>
      <xsl:call-template name="descToTitleAttribute"/>
      <xsl:call-template name="processTable"/>
    </div>
  </xsl:template>

  <xsl:template name="processTable">
    <xsl:variable name="title" select="./*[contains(@class,' topic/title ')]"/>

    <xsl:choose>
      <xsl:when test="$title">
        <xsl:choose>
          <xsl:when
            test="index-of($titleAfterList,u:classToElementName(@class)) ge 1">
            <xsl:apply-templates select="./* except $title"/>
            <xsl:apply-templates select="$title"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>

      <xsl:otherwise>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- tgroup ============================================================ -->

  <xsl:template match="*[contains(@class,' topic/tgroup ')]">
    <table>
      <xsl:call-template name="commonAttributes"/>

      <xsl:choose>
        <xsl:when test="u:isSimpleBorderTable(.)">
          <!-- ========== table having simple borders ========== -->
          <xsl:attribute name="border">1</xsl:attribute>

          <xsl:choose>
            <xsl:when test="exists(../@pgwide)">
              <!-- LIMITATION: @pgwide partially supported:
                   100%, no matter the value of @pgwide -->
              <xsl:attribute name="width">100%</xsl:attribute>
            </xsl:when>
            <xsl:when test="$default-table-width != ''">
              <xsl:attribute name="width" select="$default-table-width"/>
            </xsl:when>
          </xsl:choose>

          <!-- Display attributes -->

          <xsl:variable name="style">
            <xsl:if test="@scale">
              font-size: <xsl:value-of select="@scale"/>%;
            </xsl:if>

            <xsl:if
            test="(index-of($centerList,u:classToElementName(../@class)) ge 1) 
                  and empty(../@pgwide)">
              margin-left: auto; margin-right: auto;
            </xsl:if>
          </xsl:variable>

          <xsl:if test="$style != ''">
            <xsl:attribute name="style" select="normalize-space($style)"/>
          </xsl:if>
        </xsl:when>

        <xsl:otherwise>
          <!-- ========== table having complex borders ========== -->
          <xsl:attribute name="width">100%</xsl:attribute>
          <xsl:attribute name="cellspacing">0</xsl:attribute>

          <xsl:variable name="style">
            margin: 0;

            <!-- Display attributes -->

            <xsl:if test="@scale">
              font-size: <xsl:value-of select="@scale"/>%;
            </xsl:if>

            <xsl:variable name="isFirstTgroup" 
              select="count(preceding-sibling::*[contains(@class,' topic/tgroup ')]) eq 0"/>
            <xsl:variable name="isLastTgroup" 
              select="count(following-sibling::*[contains(@class,' topic/tgroup ')]) eq 0"/>

            <!-- No @frame means all. No @rowsep, @colsep means 1. -->
            <xsl:variable name="top" 
              select="$isFirstTgroup and
                      (../@frame = 'top' or ../@frame = 'topbot' or 
                       ../@frame = 'all' or empty(../@frame))"/>
            <xsl:variable name="bottom" 
              select="$isLastTgroup and
                      (../@frame = 'bottom' or ../@frame = 'topbot' or 
                       ../@frame = 'all' or empty(../@frame))"/>
            <xsl:variable name="left"
              select="(../@frame = 'sides' or ../@frame = 'all' or 
                       empty(../@frame))"/>
            <xsl:variable name="right" select="$left"/>

            <xsl:if test="$top">
              border-top-width: 1px;
              border-top-style: solid;
            </xsl:if>
            <xsl:if test="$bottom">
              border-bottom-width: 1px;
              border-bottom-style: solid;
            </xsl:if>
            <xsl:if test="$left">
              border-left-width: 1px; 
              border-left-style: solid; 
            </xsl:if>
            <xsl:if test="$right">
              border-right-width: 1px;
              border-right-style: solid;
            </xsl:if>
          </xsl:variable>

          <xsl:if test="$style != ''">
            <xsl:attribute name="style" select="normalize-space($style)"/>
          </xsl:if>
        </xsl:otherwise>
      </xsl:choose>

      <xsl:choose>
        <xsl:when test="number(@cols) gt 0">
          <xsl:variable name="colspecs" 
                        select="./*[contains(@class,' topic/colspec ')]"/>

          <xsl:if test="count($colspecs) gt number(@cols)">
            <xsl:message
                terminate="yes">Too many "colspec" elements.</xsl:message>
          </xsl:if>

          <xsl:if test="count($colspecs) gt 0">
            <xsl:apply-templates select="$colspecs"/>

            <xsl:for-each select="(count($colspecs)+1) to xs:integer(@cols)">
              <col class="colspec" width="1*"/>
            </xsl:for-each>
          </xsl:if>
        </xsl:when>

        <xsl:otherwise>
          <xsl:message
            terminate="yes">Missing or invalid "cols" attribute.</xsl:message>
        </xsl:otherwise>
      </xsl:choose>

      <xsl:apply-templates select="./*[contains(@class,' topic/thead ')]"/>
      <xsl:apply-templates select="./*[contains(@class,' topic/tbody ')]"/>
    </table>
  </xsl:template>

  <xsl:function name="u:isSimpleBorderTable" as="xs:boolean">
    <xsl:param name="elem" as="element()"/>

    <xsl:for-each select="$elem">
      <xsl:variable name="table"
        select="ancestor-or-self::*[contains(@class,' topic/table ')]"/>

      <xsl:choose>
        <xsl:when test="exists($table)">
          <!-- Here generate-id() is just used to key simpleBorderTable. -->
          <xsl:sequence
            select="exists(key('simpleBorderTable', generate-id($table[1])))"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:sequence select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
  </xsl:function>

  <!-- colspec =========================================================== -->

  <xsl:template match="*[contains(@class,' topic/colspec ')]">
    <col>
      <xsl:call-template name="commonAttributes"/>

      <xsl:attribute name="width">
        <xsl:choose>
          <xsl:when test="@colwidth">
            <xsl:value-of select="u:convertLength(@colwidth)"/>
          </xsl:when>
          <xsl:otherwise>1*</xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>

      <xsl:if test="@align">
        <xsl:attribute name="align" select="@align"/>
      </xsl:if>
      <xsl:if test="@char">
        <xsl:attribute name="char" select="@char"/>
      </xsl:if>
      <xsl:if test="@charoff">
        <xsl:attribute name="charoff" select="@charoff"/>
      </xsl:if>

      <xsl:apply-templates/>
    </col>
  </xsl:template>

  <xsl:function name="u:convertLength">
    <xsl:param name="length" as="xs:string"/>

    <xsl:choose>
      <xsl:when test="contains($length, '+')">
        <!-- LIMITATION: something like colwidth="2*+3pt" is not supported -->
        <xsl:sequence select="'1*'"/>
      </xsl:when>

      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test="ends-with($length, 'pt')">
            <xsl:variable name="value"
              select="number(substring-before($length, 'pt'))"/>
            <xsl:choose>
              <xsl:when test="$value gt 0">
                <xsl:sequence
                    select="round(($value div 72.0) * $screen-resolution)"/>
              </xsl:when>
              <xsl:otherwise>
                <!-- Not a number -->
                <xsl:sequence select="'1*'"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>

          <xsl:when test="ends-with($length, 'pi')">
            <xsl:variable name="value"
              select="number(substring-before($length, 'pi'))"/>
            <xsl:choose>
              <xsl:when test="$value gt 0">
                <xsl:sequence
                  select="round(($value div 6.0) * $screen-resolution)"/>
              </xsl:when>
              <xsl:otherwise>
                <!-- Not a number -->
                <xsl:sequence select="'1*'"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>

          <xsl:when test="ends-with($length, 'cm')">
            <xsl:variable name="value"
              select="number(substring-before($length, 'cm'))"/>
            <xsl:choose>
              <xsl:when test="$value gt 0">
                <xsl:sequence
                  select="round(($value div 2.54) * $screen-resolution)"/>
              </xsl:when>
              <xsl:otherwise>
                <!-- Not a number -->
                <xsl:sequence select="'1*'"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>

          <xsl:when test="ends-with($length, 'mm')">
            <xsl:variable name="value"
              select="number(substring-before($length, 'mm'))"/>
            <xsl:choose>
              <xsl:when test="$value gt 0">
                <xsl:sequence
                  select="round(($value div 25.4) * $screen-resolution)"/>
              </xsl:when>
              <xsl:otherwise>
                <!-- Not a number -->
                <xsl:sequence select="'1*'"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>

          <xsl:when test="ends-with($length, 'in')">
            <xsl:variable name="value"
              select="number(substring-before($length, 'in'))"/>
            <xsl:choose>
              <xsl:when test="$value gt 0">
                <xsl:sequence select="round($value * $screen-resolution)"/>
              </xsl:when>
              <xsl:otherwise>
                <!-- Not a number -->
                <xsl:sequence select="'1*'"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>

          <xsl:when test="ends-with($length, '*')">
            <xsl:sequence select="$length"/>
          </xsl:when>

          <xsl:otherwise>
            <xsl:variable name="value" select="number($length)"/>
            <xsl:choose>
              <xsl:when test="$value gt 0">
                <!-- Implicit pt -->
                <xsl:sequence
                  select="round(($value div 72.0) * $screen-resolution)"/>
              </xsl:when>
              <xsl:otherwise>
                <!-- Unit not supported -->
                <xsl:sequence select="'1*'"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <!-- thead ============================================================ -->

  <xsl:template match="*[contains(@class,' topic/thead ')]">
    <thead>
      <xsl:call-template name="commonAttributes"/>

      <xsl:call-template name="inheritCellAlignAttributes"/>
      <xsl:if test="@valign">
        <xsl:attribute name="valign" select="@valign"/>
      </xsl:if>

      <xsl:apply-templates/>
    </thead>
  </xsl:template>

  <xsl:template name="inheritCellAlignAttributes">
    <xsl:if test="../@align">
      <xsl:attribute name="align" select="../@align"/>
    </xsl:if>
    <xsl:if test="../@char">
      <xsl:attribute name="char" select="../@char"/>
    </xsl:if>
    <xsl:if test="../@charoff">
      <xsl:attribute name="charoff" select="../@charoff"/>
    </xsl:if>
  </xsl:template>

  <!-- tbody ============================================================ -->

  <xsl:template match="*[contains(@class,' topic/tbody ')]">
    <tbody>
      <xsl:call-template name="commonAttributes"/>

      <xsl:call-template name="inheritCellAlignAttributes"/>
      <xsl:if test="@valign">
        <xsl:attribute name="valign" select="@valign"/>
      </xsl:if>

      <xsl:apply-templates/>
    </tbody>
  </xsl:template>

  <!-- row =============================================================== -->

  <xsl:template match="*[contains(@class,' topic/row ')]">
    <tr>
      <xsl:call-template name="commonAttributes"/>

      <xsl:if test="@valign">
        <xsl:attribute name="valign" select="@valign"/>
      </xsl:if>

      <xsl:apply-templates/>
    </tr>
  </xsl:template>

  <!-- entry ============================================================= -->

  <xsl:template match="*[contains(@class,' topic/entry ')]">
    <!-- LIMITATION:
         * In order to inherit @rowsep and @colsep from a colspec, an entry
           must refer to it explicitly using its @colname or its @namest.
         * @colname or @namest cannot be used to skip columns.
    -->

    <!-- This is a simplication -->
    <xsl:variable name="isFirstColumn" select="empty(preceding-sibling::*)"/>
    <xsl:variable name="isHeader"
      select="ancestor::*[contains(@class,' topic/thead ')] or
              (ancestor::*[contains(@class,' topic/table ') and 
                           (@rowheader = 'firstcol')] and 
               $isFirstColumn)"/>

    <xsl:choose>
      <xsl:when test="$isHeader">
        <th>
          <xsl:call-template name="cellAttributes">
            <xsl:with-param name="classPrefix" select="'header-'"/>
          </xsl:call-template>
          <xsl:call-template name="processCellContents"/>
        </th>
      </xsl:when>
      <xsl:otherwise>
        <td>
          <xsl:call-template name="cellAttributes"/>
          <xsl:call-template name="processCellContents"/>
        </td>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="cellAttributes">
    <xsl:param name="classPrefix" select="''"/>

    <xsl:call-template name="commonAttributes">
      <xsl:with-param name="classPrefix" select="$classPrefix"/>
    </xsl:call-template>

    <xsl:if test="@align">
      <xsl:attribute name="align" select="@align"/>
    </xsl:if>
    <xsl:if test="@char">
      <xsl:attribute name="char" select="@char"/>
    </xsl:if>
    <xsl:if test="@charoff">
      <xsl:attribute name="charoff" select="@charoff"/>
    </xsl:if>
    <xsl:if test="@valign">
      <xsl:attribute name="valign" select="@valign"/>
    </xsl:if>

    <xsl:call-template name="cellSpanAttributes"/>

    <xsl:if test="not(u:isSimpleBorderTable(.))">
      <xsl:call-template name="cellSeparAttributes"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="cellSpanAttributes">
    <xsl:variable name="rowspan" select="u:entryRowSpan(.)"/>
    <xsl:if test="$rowspan gt 1">
      <xsl:attribute name="rowspan" select="$rowspan"/>
    </xsl:if>

    <xsl:variable name="colspan" select="u:entryColSpan(.)"/>
    <xsl:if test="$colspan gt 1">
      <xsl:attribute name="colspan" select="$colspan"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="cellSeparAttributes">
    <!-- Do not use //*: a table can contain another table. -->
    <xsl:variable name="rows" 
      select="ancestor::*[contains(@class,' topic/table ')]/*/*/*[contains(@class,' topic/row ')]"/>
    <xsl:variable name="parent" select="parent::*"/>
    <xsl:variable name="row" select="u:indexOfNode($rows, $parent)" />

    <xsl:variable name="rowCount" select="count($rows)"/>

    <xsl:variable name="nextRow" select="$row + u:entryRowSpan(.)"/>

    <xsl:variable name="colspec" select="u:entryColspec(.)"/>

    <xsl:variable name="rowsep">
      <xsl:choose>
        <xsl:when test="$nextRow gt $rowCount">0</xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <!-- entry -->
            <xsl:when test="@rowsep">
              <xsl:value-of select="@rowsep"/>
            </xsl:when>
            <!-- row -->
            <xsl:when test="../@rowsep">
              <xsl:value-of select="../@rowsep"/>
            </xsl:when>
            <!-- colspec -->
            <xsl:when test="exists($colspec) and $colspec/@rowsep">
              <xsl:value-of select="$colspec/@rowsep"/>
            </xsl:when>
            <!-- tbody has no rowsep attribute -->
            <!-- tgroup -->
            <xsl:when test="../../../@rowsep">
              <xsl:value-of select="../../../@rowsep"/>
            </xsl:when>
            <!-- table -->
            <xsl:when test="../../../../@rowsep">
              <xsl:value-of select="../../../../@rowsep"/>
            </xsl:when>
            <xsl:otherwise>1</xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="colsep">
      <xsl:choose>
        <!-- This is a simplication -->
        <xsl:when test="count(following-sibling::*) eq 0">0</xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <!-- entry -->
            <xsl:when test="@colsep">
              <xsl:value-of select="@colsep"/>
            </xsl:when>
            <!-- colspec -->
            <xsl:when test="exists($colspec) and $colspec/@colsep">
              <xsl:value-of select="$colspec/@colsep"/>
            </xsl:when>
            <!-- row and tbody have no colsep attribute -->
            <!-- tgroup -->
            <xsl:when test="../../../@colsep">
              <xsl:value-of select="../../../@colsep"/>
            </xsl:when>
            <!-- table -->
            <xsl:when test="../../../../@colsep">
              <xsl:value-of select="../../../../@colsep"/>
            </xsl:when>
            <xsl:otherwise>1</xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="style">
      <xsl:if test="$rowsep != '0'">
        border-bottom-width: 1px;
        border-bottom-style: solid;
      </xsl:if>
      <xsl:if test="$colsep != '0'">
        border-right-width: 1px;
        border-right-style: solid;
      </xsl:if>
    </xsl:variable>

    <xsl:if test="$style != ''">
      <xsl:attribute name="style" select="normalize-space($style)"/>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet>
